@@ -3,11 +3,8 @@
 import ee
 import ee.data
 
-if not ee.data._initialized:
-    ee.Initialize()
 
-
-def trim_decimals(number, places=2):
+def trimDecimals(number, places=2):
     """ Decrease the number of decimals in a ee.Number
 
     :param places: number of decimal places to leave
