@@ -9,6 +9,7 @@ import os.path
 import ee
 import logging
 
+
 def listEE2list(listEE, type='Image'):
     relation = {'Image': ee.Image,
                 'Number': ee.Number,
@@ -21,6 +22,7 @@ def listEE2list(listEE, type='Image'):
 
     return newlist
 
+
 class ImageStrip(object):
     ''' Create an image strip '''
     def __init__(self, name, ext="png", description="", **kwargs):
@@ -229,8 +231,8 @@ class ImageStrip(object):
 
         return strip
 
-    def from_list(self, imlist, namelist, viz_bands, min, max, region,
-                  folder, check=True, desclist=None, **kwargs):
+    def fromList(self, imlist, namelist, viz_bands, min, max, region,
+                 folder, check=True, desclist=None, **kwargs):
         """ Download every image and create the strip
 
         :param imlist: Satellite Images (not PIL!!!!!!)
@@ -364,9 +366,9 @@ class ImageStrip(object):
         else:
             return self.create(list_of_lists, namelist)
 
-    def from_collection(self, collections, viz_param=None, region=None,
-                        name=None, folder=None, properties=None,
-                        drawRegion=False, zoom=0, description="", **kwargs):
+    def fromCollection(self, collections, viz_param=None, region=None,
+                       name=None, folder=None, properties=None,
+                       drawRegion=False, zoom=0, description="", **kwargs):
         ''' Create an image strip from a given ee.ImageCollection or ee.List
 
         :param collections: contains the images (ee.Image)
@@ -510,6 +512,6 @@ class ImageStrip(object):
 
         # print arguments["draw_polygons"]
 
-        i = pl.from_list(**arguments)
+        i = pl.fromList(**arguments)
 
         return i
\ No newline at end of file
